import numpy as np
from prob_theory import random_string

import tree_and_grammar.theory_calls
from graphs.grammar import Grammar
from search_methods import bfs_search, dfs_search


class BinaryGrammar(Grammar):
    """Model investigated in paper as Binary Grammar"""

    def __init__(self, depth=3, g=2, pg=0.25, create_graph=True):
        self.depth = depth
        self.g = g
        self.pg = pg
        rules = [('', 'a'), ('', 'b')]
        start_string = ''
        # This sampling of goals is equivalent to iid sampling with prob p
        num_goals = np.random.binomial(2**g, pg)
        print("num_goals", num_goals)
        goals = [random_string(['a', 'b'], g) for j in range(num_goals)]
        Grammar.__init__(self, rules=rules, max_depth=depth,
                         start_string=start_string, goal_strings=goals,
                         create_graph=create_graph)

    def expected_dfs(self):
        return (tree_and_grammar.theory_calls.tdfs_l_clust(self.depth, self.g, self.pg) +
                tree_and_grammar.theory_calls.tdfs_u_clust(self.depth, self.g, self.pg)) / 2

    def expected_dfsl(self):
        return tree_and_grammar.theory_calls.tdfs_l_clust(self.depth, self.g, self.pg)

    def expected_dfsu(self):
        return tree_and_grammar.theory_calls.tdfs_u_clust(self.depth, self.g, self.pg)

    def expected_bfs(self):
        return tree_and_grammar.theory_calls.tbfs_clust(self.depth, self.g, self.pg)

    def goal_depth(self):
        return self.g

    def goal_prob(self):
        return self.pg

    def predicted_difference(self):
        return self.expected_dfs() - self.expected_bfs()

    def bfs_better_theory(self):
        return self.expected_dfs() > self.expected_bfs()

    feature_list = [
        bfs_search,
        dfs_search,
        expected_dfs,
        expected_dfsl,
        expected_dfsu,
        expected_bfs,
    ]