
import random
import math

from graphs.complete_tree import CompleteTree
from search_methods import bfs_search, dfs_search

class CompleteTreeMGL(CompleteTree):
    """Gaussian multi-goal level model"""

    def __init__(self, depth=5, peak=3, spread=1, create_graph=True):
        self.K = peak
        self.s = spread
        goal_prob = [min(np.exp(-(peak-k)**2/spread) *
                         0.05/np.sqrt(spread), 1/2)
                     for k in range(depth+1)]
        CompleteTree.__init__(self, goal_prob=goal_prob,
                              create_graph=create_graph)

    def goal_peak(self):
        return self.K

    def goal_spread(self):
        return self.s

    @staticmethod
    def random_init():
        depth  = random.randint(7,15)
        middle = math.floor(depth/2)
        peak   = random.randint(middle-3, middle+3)
        return {
            depth    : depth,
            peak     : peak,
            spread   : 1
        }

    feature_list = [
        CompleteTree.depth,
        #goal_peak,
        #goal_spread,
        #max_prob,
        CompleteTree.expected_bfs,
        bfs_search,
        CompleteTree.expected_dfs,
        dfs_search,
        CompleteTree.prediction_correct
    ]