import math
import random

from graphs.complete_tree import CompleteTree
from search_methods import bfs_search, dfs_search
from theory.prob_theory import gamma, expectation_truncated_geometric


class CompleteTreeSGL(CompleteTree):
    """Single goal level model"""

    def __init__(self, depth=5, goal_depth=3,
                 goal_prob=0.15, create_graph=True):
        assert int(depth) and int(goal_depth) and float(goal_prob)
        assert depth >= goal_depth >= 0
        assert 0 <= goal_prob <= 1
        goal_prob_vec = [0 for i in range(depth+1)]
        goal_prob_vec[math.ceil(goal_depth)] = goal_prob
        CompleteTree.__init__(self, goal_prob=goal_prob_vec,
                              create_graph=create_graph)
        self.K = goal_depth
        self.p = goal_prob

    def goal_depth(self):
        return self.K

    def goal_depth_quote(self):
        return self.K/self.D

    def goal_prob(self):
        return self.p

    def bfs_better(self):  # as predicted by theory
        # gamma = 1 + np.log2((1-self.p)/self.p)
        gammap = gamma(self.p, self.K)
        K = self.K
        D = self.D
        return (K < D/2 + gammap/2)

    def sgl_expected_bfs(self):
        G = self.has_goal()
        return G*(2 ** self.K - 1 +
                  expectation_truncated_geometric(self.p, 2 ** self.K)) + \
            (1-G)*2**(self.D+1)

    def sgl_expected_dfs(self):
        G = self.has_goal()
        return G*((expectation_truncated_geometric(self.p, 2 ** self.K) - 1) *
                  2 ** (self.D-self.K+1) + 2) +\
            (1-G)*2**(self.D+1)

    def marcus_exp_dfs(self):
        return (1/self.p - 1)*2**(self.D-self.K+1) + 2

    feature_list = [
        bfs_search,
        dfs_search,
        #number_of_goals,
        CompleteTree.depth,
        CompleteTree.expected_dfs,
        #marcus_exp_dfs,
        #goal_depth,
        #goal_prob,
        #CompleteTree.expected_bfs,
        #CompleteTreeSGL.goal_depth,
        lambda g: int(bfs_search(g) < dfs_search(g)),
        #bfs_better
    ]

    @staticmethod
    def random_init():
        depth = random.randint(4, 15)
        goal_depth = random.randint(3, depth)
        #goal_prob = random.uniform(0.01, 0.5)
        #depth = 16
        #goal_depth = 12
        #goal_depth=depth-3
        goal_prob = 0.07
        return {
            'depth'      : depth,
            'goal_depth' : goal_depth,
            'goal_prob'  : goal_prob
        }