import random

import numpy as np

from graphs.n_puzzle import NPuzzleGraph
from theory.prob_theory import discrete_gaussian


class NPuzzleGaussianGoals(NPuzzleGraph):
    """NPuzzleGraph with goals distributed according to Gaussian probability"""

    def __init__(self, mu, sd, goal_freq, n=3):
        NPuzzleGraph.__init__(self, n=3)
        self.mu = mu
        self.sd = sd
        self.make_gaussian_goals(mu=mu, sd=sd, goal_freq=goal_freq)
        self.goal_freq = goal_freq

    def make_gaussian_goals(self, mean_level, goal_freq=1, sd=3.5, verbose=False):
        """Set the expected number of goals with center at goal level"""
        self.goal_freq = goal_freq
        self.root = self.vertex(0)
        self.goals = []
        for v in self.vertices():
            d = self.vp.depth[v]
            if random.random() < goal_freq * discrete_gaussian(d, sd=sd, mu=mean_level):
                # discrete_gaussian(d, sd=sd, mu=mean_level)/bg**d:
                NPuzzleGraph.make_goal(self, v)
            else:
                self.vp.goal[v] = False
        if verbose:
            print("Graph gets", len(self.goals), "goals")

    @staticmethod
    def goal_distribution(d, goal_freq=1, D=20):
        pv = [goal_freq * discrete_gaussian(x, mu=d, sd=3.5) for x in range(D + 1)]
        return np.array(pv).clip(0, 1)
