import numpy as np

from graphs.n_puzzle import NPuzzleGraph
from n_puzzle import n_puzzle_params
from theory.prob_theory import discrete_gaussian


class NPuzzleMTGoals(NPuzzleGraph):
    """NPuzzleGraph with all goals having at most threshold misplaced tiles considered goals"""

    def __init__(self, threshold, n=3):
        NPuzzleGraph.__init__(self, n=3)
        self.mt_threshold = threshold
        self.make_mt_goals(threshold=threshold)
        self.mt_nodes = self.nodes_of_mt()
        self.mtd = {i: self.mt_nodes[i] for i in self.mt_nodes}

    def make_mt_goals(self, threshold=3):
        self.mt_threshold = threshold
        for v in self.vertices():
            if self.vp.mt[v] <= threshold:
                NPuzzleGraph.make_goal(self, v)
            else:
                self.vp.goal[v] = False

    @staticmethod
    def goal_distribution_mt(mh, D=20, mt=3):
        """Estimated goal distribution from Manhattan distance"""
        number_of_goals = sum(n_puzzle_params.mt_nodes[:mt + 1])
        sd = 2.5
        mu = 1.5*mh
        pv = np.array([number_of_goals * discrete_gaussian(i, mu=mu, sd=sd) / n_puzzle_params.bgdown ** i
                       for i in range(D+1)])
        return pv.clip(0, 1)