import numpy as np
import random

from graphs.n_puzzle import NPuzzleGraph


class NPuzzleSGLGoals(NPuzzleGraph):
    """NPuzzleGraph with all goals having at most threshold misplaced tiles considered goals"""

    def __init__(self, goal_level, goal_probability):
        NPuzzleGraph.__init__(self, n=3)
        self.goal_level = goal_level
        self.goal_probability = goal_probability
        self.add_sgl_goals(goal_level, gp=goal_probability)

    def add_sgl(self, level, gp=0.1):
        """Set the expected number of goals with center at goal level"""
        for v in self.vertices():
            d = self.vp.depth[v]
            if d == int(level) and random.random() < gp:
                NPuzzleGraph.make_goal(self, v)
            else:
                self.vp.goal[v] = False

    def goal_distribution(self, d, D=20, gp=None):
        if not gp:
            gp = self.goal_probability
        pv = [0 for x in range(D + 1)]
        if d in range(len(pv)):
            pv[d] = gp
        return np.array(pv)