import random

from graphs.goal_graph import GoalGraph
from graphs.grammar import Grammar
from search_methods import bfs_search, dfs_search
from theory.prob_theory import random_string


class RandomGrammar(Grammar):

    def __init__(self,
                 num_rules        = 3,  # number of rules in addition to S->''
                 num_goal_strings = 3,  # number of goal strings
                 len_goal_strings = lambda: random.randint(1, 3), #length of goal strings
                 max_depth        = 4):  # maximum depth of tree
        rules = [ ('S', '') ]
        possible_rules = [
            ('S', 'Sa'),
            ('S', 'Sb'),
            ('S', 'aS'),
            ('S', 'bS'),
            ('Sa', 'aS'),
            ('Sb', 'bS'),
            ('aS', 'Sa'),
            ('bS', 'Sb'),
        ]
        rules.extend(random.sample(possible_rules, num_rules))
        goals = [random_string(['a', 'b'], len_goal_strings())
                 for j in range(num_goal_strings)]
        Grammar.__init__(self,
                         start_string = 'S',
                         goal_strings = goals,
                         rules        = rules,
                         max_depth    = max_depth)

    @staticmethod
    def random_init():
        max_depth        = 15 #random.randint(11,15)
        num_goal_strings = random.randint(3, 5) # *max_depth)
        lgl              = 2 #random.randint(1, 3)
        ugl              = 5 #random.randint(lgl, 5) # max_depth)
        len_goal_strings = lambda: random.randint(lgl, ugl)
        num_rules        = random.randint(3, 8)
        return dict(
            max_depth        = max_depth,
            num_goal_strings = num_goal_strings,
            len_goal_strings = len_goal_strings,
            num_rules        = num_rules
        )

    feature_list = [
        #GoalGraph.degree_mean,
        #GoalGraph.degree_stdev,
        #Grammar.number_of_rules,
        #Grammar.maximum_depth,
        #Grammar.num_goal_strings,
        #Grammar.mean_len_goal_str,
        #Grammar.stdev_len_goal_str,
        # Questionable features:
        #Grammar.grammar_power,
        GoalGraph.number_of_goals,
        #GoalGraph.shallowest_goal,
        #GoalGraph.pseudo_diameter,
        #GoalGraph.goal_frequency,
        bfs_search,
        dfs_search,
        #GoalGraph.bfs_beats_dfs,
        #GoalGraph.dfs_minus_bfs,
    ]