import random

from graph_tool import generation, topology

from graphs.goal_graph import GoalGraph
from theory.prob_theory import sample_k


class RandomGraph(GoalGraph):
    """Create a goal graph with random structure and
    randomly sampled goals at specified distance from root"""

    def __init__(self,
                 N=40,
                 goal_level=3,
                 number_of_goals=2,
                 degree=lambda: sample_k(5)):
        GoalGraph.__init__(self)
        self.K = goal_level
        self.number_of_goals = number_of_goals
        # Copy a random graph to self
        rgraph = generation.random_graph(N, degree, directed=False)
        self.add_vertex(N)
        for e in rgraph.edges():
            self.add_edge(e.source().__int__(), e.target().__int__())
        # Put goals at the right level (rejection sampling, inefficient)
        # could be made more efficient by BFS layering the graph
        g = 0
        i = 0
        while g < self.number_of_goals and i < 10*N:
            i += 1
            v = self.vertex(random.randint(1, N))
            if topology.shortest_distance(self, self.root, v)==goal_level and \
               not self.goal[v]:
                self.goal[v] = True
                self.goals.append(v)
                g += 1
        if i >= 10*N:
            print("goaling failed")

    @staticmethod
    def random_init():
        N=random.randint(10,100)
        goal_level=random.randint(3,max(3,int(N/10)))
        number_of_goals=random.randint(1,goal_level)
        degree=lambda: sample_k(5)
        return {
            N               : N,
            goal_level      : goal_level,
            number_of_goals : number_of_goals,
            degree          : degree
        }

    # feature_list = [
    #     number_of_goals,
    #     pseudo_diameter,
    #     Goalshallowest_goal,
    #     degree_mean,
    #     degree_stdev,
    #     bfs_search,
    #     dfs_search,
    #     lambda g: bfs_search(g)-dfs_search(g)
    # ]