import random
import sys

import graph_tool

from n_puzzle import n_puzzle_params


def random_walk_no_return(G, max_length=40, verbose=False):
    len2depth = {}  # path length -> depth
    visited = set()

    def rec(u, length=0):
        visited.add(u)
        len2depth[length] = G.vp.depth[u]
        children = set(u.out_neighbours()) - visited
        if len(children) == 0:
            if verbose:
                print("path died at length ", length, file=sys.stderr)
            return
        v = random.choice(list(children))
        if length < max_length:
            rec(v, length=length+1)
        else:
            return

    rec(G.root)
    return len2depth


def analyze_search_depth(G, iterations=100, max_length=20):
    max_depth = n_puzzle_params.max_depth
    res = {i: [0 for j in range(max_depth)]
           for i in range(max_length+1)}  # length -> [depth1, depth2, ...]
    for i in range(iterations):
        len2depth = random_walk_no_return(G, max_length=max_length)
        for l, d in len2depth.items():
            res[l][d] += 1/iterations
    return res


if __name__ == "__main__":
    G = graph_tool.load_graph("new_data/n3graph.xml.gz")
    G.root = G.vertex(0)
    analyze_search_depth(G, iterations=100, max_length=20)
