import pickle

import graph_tool
import pandas

from graphs.n_puzzle import NPuzzleGraph
from graphs.n_puzzle_art_goals import NPuzzleArtGoals
from search_methods import BFS_graph, DFS_tree


def search_time_art(G, searchers, goal_freq=1, D=20, iterations=10, verbose=False):
    """
    Generate data frame with search times for supplied search method
    for different Manhattan distances.
    Use plot_search_time function to visualise.
    """
    data = {s: [] for s in searchers}
    mh_nodes = NPuzzleGraph.nodes_of_mh(G)
    for mh in mh_nodes:
        for i in range(iterations):
            NPuzzleArtGoals.make_art_goals(G, mh, goal_freq)
            for s in searchers:
                (s_time, visited, goal_found) = searchers[s](G, max_path_length=D, verbose=verbose)
                data[s].append([mh, s_time, visited, goal_found])
                if verbose:
                    print(mh, i, s, data[s][-1])
    return {s: pandas.DataFrame(data[s], columns=("mh", "emp", "visited", "goal found")) for s in searchers}


# def search_time_all_gauss(G, number_of_goals=1, iterations=10, verbose=False):
#     bfs_df = search_time_gauss(G, number_of_goals, BFS_graph, iterations=iterations, verbose=verbose)
#     print("BFS done")
#     dfs_df = search_time_gauss(G, number_of_goals, DFS_tree, iterations=iterations, verbose=verbose)
#     print("DFS done")
#     # dfsg_df = iterate_DFSg(G, iterations=iterations)
#     return {'BFS': bfs_df, 'DFSt': dfs_df}   # , 'DFSg': dfsg_df}


if __name__ == "__main__":
    G = graph_tool.load_graph("new_data/n3graph.xml.gz")
    runtimes = search_time_art(G, {'BFS': BFS_graph, 'DFS': DFS_tree}, iterations=100, verbose=True)
    pickle.dump(runtimes, open("new_data/runtimes_gauss.p", "wb"))