import numpy as np
from matplotlib import pyplot as plt
from prob_theory import nCk

from theory.counters2searchtime import tbfs, upper_tdfs, lower_tdfs


def tbfs_extra_level(D, p, L):
    return tbfs(D, p, L, extra_level=True)


def tdfs_u_clust(D, g, p):
    pv = [0]*(D+1)
    pv[g] = p
    return upper_tdfs(D, pv, l_grammar_clustered)


def tdfs_l_clust(D, g, p):
    pv = [0]*(D+1)
    pv[g] = p
    return lower_tdfs(D, pv, l_grammar_clustered)


def tbfs_clust(D, g, p):
    pv = [0]*(D+1)
    pv[g] = p
    return tbfs(D, pv, l_grammar_clustered)


def l_grammar_clustered(D, n, d):
    """Number of clustered nodes at level d reachable from a^n"""
    assert n >= 0
    return sum([nCk(d, i) for i in range(d - n + 1)])


def l_grammar_unclustered(D, n, d):
    """Number of unclustered nodes at level d reachable from a^n"""
    if d == 0:
        return int(n == d)
    elif d <= D:
        return (d+1)*l_grammar_clustered(D, n, d) +\
            l_grammar_clustered(D, n, d-1)
    elif d == D+1:
        return l_grammar_clustered(D, n, D)
    else:
        return 0


def l_complete_binary(n, d):
    """Number of nodes at level d reachable from a^n in complete binary tree"""
    if d >= n:
        return 2**(d-n)
    else:
        return 0


def expected_dfs(D, p, L):
    """For complete binary tree model"""
    if max(p) == 1:
        return np.argmax(p)+1
    # level rates
    l = [-np.log(1-pk) for pk in p]
    # total rate
    tr = sum([lk/(2**(D-k+1)) for (k, lk) in enumerate(l)])
    return 1/max(tr, 2**(-D))


def plot(D, pv, L, tv):
    res = np.zeros((len(pv), len(tv)))
    for (k, p) in enumerate(pv):
        print(p)
        res[k, :] = [t(D, p, L) for t in tv]
    plt.plot(res)
    plt.show()
    return res



# D = 15
# pv = np.zeros((D+1, D+1))
# for k in range(2,D+1):
#     pv[k,k] = 0.001 #1/2**(k-1)


assert l_grammar_unclustered(2, 0, 3) == 4
assert l_grammar_unclustered(2, 0, 2) == 14
assert l_grammar_unclustered(2, 0, 1) == 5
assert l_grammar_unclustered(2, 0, 0) == 1

assert l_grammar_unclustered(2, 1, 3) == 3
assert l_grammar_unclustered(2, 1, 2) == 10
assert l_grammar_unclustered(2, 1, 1) == 2
assert l_grammar_unclustered(2, 1, 0) == 0

assert l_grammar_unclustered(2, 2, 3) == 1
assert l_grammar_unclustered(2, 2, 2) == 3
assert l_grammar_unclustered(2, 2, 1) == 0
assert l_grammar_unclustered(2, 2, 0) == 0